/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "leveldiagram.h"
#include <stdio.h>

void
LevelDiagram::generateDiagram(Diagram::generationParams_t *params)
{
  /*
    We first place minimal elements with increasing evaluation.
    Then symmetrically above lower level of elements we place their 
    upper ones, evaluation of each upper element is the sum of 
    evaluations of his lower ones and elements are on the level sorted 
    by their evaluation which must be increasing.
  */
  Eval ev;
  set<Eval> evals1, *evals_l, evals2, *evals_u, *evals3;
  set<ElementId> elems1, elems2;
  set<ElementId>::iterator iter_sel;
  set<Eval>::iterator iter_sev1, iter_sev2;
  eval_t num = 0;
  Vertex_e vert;
  Coord_e_t x, y = 0;
  set<Vertex_e>::iterator iter_sv;
  set<Vertex_e> verts;

  if (!params) return;
  generationParams = (generationParams_t *)params;
  evals_l = &evals1; evals_u = &evals2;
  data->getElements(elems2);
  if (generationParams->from_minimal) data->minimalElements(elems1, elems2);
  else data->maximalElements(elems1, elems2);
  x = -elems1.size() + 1;
  // TODO: another order of processing
  for (iter_sel = elems1.begin(); iter_sel != elems1.end(); iter_sel++) {
    ev.element = *iter_sel; ev.eval = num++;
    evals_l->insert(ev);
    vert.element = *iter_sel; vert.dummy = NoElement;
    vert.x = x; x += 2; vert.y = y;
    verts.insert(vert);
  }
  while (evals_l->size()) {
    for (iter_sev1 = evals_l->begin(), num = 0; iter_sev1 != evals_l->end(); 
	 iter_sev1++) {
      elems1.clear();
      if (generationParams->from_minimal) 
	data->upperElements(elems1, (*iter_sev1).element);
      else data->lowerElements(elems1, (*iter_sev1).element);
      // TODO: another order of processing
      for (iter_sel = elems1.begin(); iter_sel != elems1.end(); 
	   iter_sel++, num++) {
      	ev.element = *iter_sel; ev.eval = (*iter_sev1).eval;
	for (iter_sev2 = evals_u->begin(); iter_sev2 != evals_u->end(); 
	     iter_sev2++)
	  if ((*iter_sev2).element == ev.element) break;
	if (iter_sev2 != evals_u->end()) { 
	  ev.eval += (*iter_sev2).eval;
	  evals_u->erase(iter_sev2);
	}
	if (ev.eval <= num) ev.eval = num;
	num = ev.eval;
	evals_u->insert(ev);
      }
    }
    x = -evals_u->size() + 1;
    if (generationParams->from_minimal) y += 2; else y -= 2;
    for (iter_sev1 = evals_u->begin(); iter_sev1 != evals_u->end(); 
	 iter_sev1++) {
      vert.element = (*iter_sev1).element; vert.dummy = NoElement;
      vert.x = x; x += 2; vert.y = y;
      iter_sv = verts.find(vert);
      if (iter_sv != verts.end()) verts.erase(iter_sv);
      verts.insert(vert);
    }
    evals3 = evals_l; evals_l = evals_u; evals_u = evals3; evals_u->clear();
  }
  verteces = verts;
}
